@@ -9,7 +9,12 @@
 #global OU (OU1), multiple regime OU (OUM), multiple sigmas (OUMV), multiple alphas (OUMA),
 #and the multiple alphas and sigmas (OUMVA).
 
-OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA", "TrendyM", "TrendyMS"), simmap.tree=FALSE, root.age=NULL, scaleHeight=FALSE, root.station=FALSE, get.root.theta=FALSE, shift.point=0.5, clade=NULL, mserr="none", starting.vals=NULL, check.identify=TRUE, diagn=FALSE, quiet=FALSE, warn=TRUE, opts = list("algorithm"="NLOPT_LN_SBPLX", "maxeval"="1000", "ftol_rel"=.Machine$double.eps^0.5)){
+OUwie <- function(phy, data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA", "TrendyM", "TrendyMS"), simmap.tree=FALSE, root.age=NULL, scaleHeight=FALSE, root.station=FALSE, get.root.theta=FALSE, shift.point=0.5, clade=NULL, mserr="none", starting.vals=NULL, check.identify=TRUE, algorithm=c("invert", "three.point"), diagn=FALSE, quiet=FALSE, warn=TRUE, opts = list("algorithm"="NLOPT_LN_SBPLX", "maxeval"="1000", "ftol_rel"=.Machine$double.eps^0.5)){
+
+    if(length(algorithm) == 2){
+        algorithm = "invert"
+        warning("An algorithm was not specified. Defaulting to computing the determinant and inversion of the vcv.", call.=FALSE, immediate.=TRUE)
+    }
 
     if(model=="BMS" & root.station==TRUE){
         warning("By setting root.station=TRUE, you have specified the group means model of Thomas et al. 2006", call.=FALSE, immediate.=TRUE)
@@ -20,12 +25,6 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
         stop("Check the format of the data column. It's reading as a factor.", call. = FALSE)
     }
 
-    #if(is.null(root.age)){
-    #    if(any(branching.times(phy)<0)){
-    #        stop("Looks like your tree is producing negative branching times. Must input known root age of tree.", call. = FALSE)
-    #    }
-    #}
-
     if(!is.null(starting.vals[1])){
         if(model == "OU1" | model == "OUM" | model == "OUMV" | model == "OUMA" | model == "OUMVA"){
             if(length(starting.vals)<2){
@@ -53,10 +52,11 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
             }
     }
 
+
     #Makes sure the data is in the same order as the tip labels
-    if(mserr == "none" | mserr == "est"){
-		data<-data.frame(data[,2], data[,3], row.names=data[,1])
-		data<-data[phy$tip.label,]
+    if(mserr == "none"){
+		data <- data.frame(data[,2], data[,3], row.names=data[,1])
+		data <- data[phy$tip.label,]
 	}
 	if(mserr=="known"){
 		if(!dim(data)[2]==4){
@@ -77,15 +77,15 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 	ntips <- length(phy$tip.label)
 	#Will label the clade of interest if the user so chooses:
 	if(!is.null(clade) & simmap.tree==FALSE){
-		node<-mrca(phy)[clade[1],clade[2]]
-		int<-c(node,Descendants(phy,node, "all"))
-		tips<-int[int<ntips]
-		data[,1]<-1
-		data[tips,1]<-2
-		int<-int[int>ntips]
-		phy$node.label<-rep(1,phy$Nnode)
-		pp<-int-length(phy$tip.label)
-		phy$node.label[pp]<-2
+		node <- mrca(phy)[clade[1],clade[2]]
+		int <- c(node,Descendants(phy,node, "all"))
+		tips <- int[int<ntips]
+		data[,1] <- 1
+		data[tips,1] <- 2
+		int <- int[int>ntips]
+		phy$node.label <- rep(1,phy$Nnode)
+		pp <- int-length(phy$tip.label)
+		phy$node.label[pp] <- 2
 	}
 	if (is.character(model)) {
 		if (model == "BM1"| model == "OU1"){
@@ -104,7 +104,8 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 		edges=cbind(c(1:(n-1)),phy$edge, MakeAgeTable(phy, root.age=root.age))
 
 		if(scaleHeight==TRUE){
-			edges[,4:5]<-edges[,4:5]/max(MakeAgeTable(phy, root.age=root.age))
+            Tmax <- max(MakeAgeTable(phy, root.age=root.age))
+			edges[,4:5]<-edges[,4:5]/Tmax
             root.age = 1
 		}
 		edges=edges[sort.list(edges[,3]),]
@@ -130,7 +131,8 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 				#New tree matrix to be used for subsetting regimes
 				edges=cbind(c(1:(n-1)),phy$edge,MakeAgeTable(phy, root.age=root.age))
 				if(scaleHeight==TRUE){
-					edges[,4:5] <- edges[,4:5]/max(MakeAgeTable(phy, root.age=root.age))
+                    Tmax <- max(MakeAgeTable(phy, root.age=root.age))
+                    edges[,4:5]<-edges[,4:5]/Tmax
                     root.age <- 1
 				}
 				edges=edges[sort.list(edges[,3]),]
@@ -157,7 +159,8 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 				#New tree matrix to be used for subsetting regimes
                 edges <- cbind(c(1:(n-1)),phy$edge,MakeAgeTable(phy, root.age=root.age))
 				if(scaleHeight==TRUE){
-					edges[,4:5] <- edges[,4:5]/max(MakeAgeTable(phy, root.age=root.age))
+                    Tmax <- max(MakeAgeTable(phy, root.age=root.age))
+                    edges[,4:5]<-edges[,4:5]/Tmax
                     root.age <- 1
 				}
 				edges=edges[sort.list(edges[,3]),]
@@ -178,78 +181,98 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 
     #Initializes an integer specifying whether or not the trendy model is to be invoked. Assume 0, unless otherwise stated later:
 	trendy <- 0
-	#Data:
-	x <- as.matrix(data[,2])
+	
+    #Data:
+    if(algorithm == "three.point"){
+        x <- data[,2]
+        names(x) <- rownames(data)
+    }else{
+        x <- as.matrix(data[,2])
+    }
 
-	#Matches the model with the appropriate parameter matrix structure
+    #Matches the model with the appropriate parameter matrix structure
 	if (is.character(model)) {
 		index.mat <- matrix(0, 2,k)
-		Rate.mat <- matrix(1, 2, k)
+        if(algorithm == "three.point"){
+            Rate.mat <- matrix(1, 3, k)
+        }else{
+            Rate.mat <- matrix(1, 2, k)
+        }
 		if (model == "BM1"){
 			np <- 1
-			index <- matrix(TRUE,2,k)
-			if(mserr=="est"){
-				index.mat[1,1:k] <- np+2
-			}
-			else{
-				index.mat[1,1:k] <- np+1
-			}
+            index.mat[1,1:k] <- NA
 			index.mat[2,1:k] <- 1
+            if(algorithm == "three.point"){
+                index.mat <- rbind(index.mat, rep(NA,k))
+            }
 			param.count <- np+1
 		}
 		#The group mean model of Thomas et al, is trivial: set root.station to be TRUE:
 		if (model == "BMS"){
 			np <- k
-			index <- matrix(TRUE,2,k)
-			if(mserr=="est"){
-				index.mat[1,1:k] <- np+2
-			}
-			else{
-				index.mat[1,1:k] <- np+1
-			}
-			index.mat[2,1:k]<-1:np
+            index.mat[1,1:k] <- NA
+			index.mat[2,1:k] <- 1:np
 			if(root.station==TRUE){
+                if(algorithm == "three.point"){
+                    max.par.so.far <- max(index.mat)
+                    index.mat <- rbind(index.mat, (max.par.so.far + 1):(max.par.so.far + k))
+                }
 				param.count <- np+k
 			}
             if(root.station==FALSE){
+                if(algorithm == "three.point"){
+                    index.mat <- rbind(index.mat, rep(NA,k))
+                }
                 param.count <- np+1
             }
 		}
 		if (model == "OU1"){
 			np <- 2
-			index <- matrix(TRUE,2,k)
 			index.mat[1,1:k] <- 1
 			index.mat[2,1:k] <- 2
+            if(algorithm == "three.point"){
+                max.par.so.far <- max(index.mat)
+                index.mat <- rbind(index.mat, rep(3,k))
+            }
             param.count <- np + 1
             if(get.root.theta == TRUE){
                 param.count <- param.count + 1
             }
 		}
 		if (model == "OUM"){
-			np <- 2
-			index <- matrix(TRUE,3,k)
+            np <- 2
 			index.mat[1,1:k] <- 1
 			index.mat[2,1:k] <- 2
-            param.count<-np+k
+            if(algorithm == "three.point"){
+                max.par.so.far <- max(index.mat)
+                index.mat <- rbind(index.mat, (max.par.so.far + 1):(max.par.so.far + k))
+            }
+            param.count <- np + k
             if(get.root.theta == TRUE){
                 param.count <- param.count + 1
             }
 		}
 		if (model == "OUMV") {
 			np <- k+1
-			index <- matrix(TRUE,2,k)
 			index.mat[1,1:k] <- 1
 			index.mat[2,1:k] <- 2:(k+1)
-            param.count <- np+k
+            if(algorithm == "three.point"){
+                max.par.so.far <- max(index.mat)
+                index.mat <- rbind(index.mat, (max.par.so.far + 1):(max.par.so.far + k))
+            }
+            param.count <- np + k
             if(get.root.theta == TRUE){
                 param.count <- param.count + 1
             }
 		}
 		if (model == "OUMA") {
 			np <- k+1
-			index <- matrix(TRUE,2,k)
 			index.mat[1,1:k] <- 1:k
 			index.mat[2,1:k] <- k+1
+            if(algorithm == "three.point"){
+                max.par.so.far <- max(index.mat)
+                index.mat <- rbind(index.mat, (max.par.so.far + 1):(max.par.so.far + k))
+            }
             param.count <- np+k
             if(get.root.theta == TRUE){
                 param.count <- param.count + 1
@@ -257,8 +280,12 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 		}
 		if (model == "OUMVA") {
 			np <- k*2
-			index <- matrix(TRUE,2,k)
+            index <- matrix(TRUE,2,k)
 			index.mat[index] <- 1:(k*2)
+            if(algorithm == "three.point"){
+                max.par.so.far <- max(index.mat)
+                index.mat <- rbind(index.mat, (max.par.so.far + 1):(max.par.so.far + k))
+            }
             param.count <- np+k
             if(get.root.theta == TRUE){
                 param.count <- param.count + 1
@@ -266,15 +293,8 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 		}
 		if (model == "TrendyM"){
 			index.mat <- matrix(0,3,k)
-			Rate.mat <- matrix(1, 3, k)
-			np=k+2 #We have k regimes, but 1 sigma, plus the root value
-			index <- matrix(TRUE,3,k)
-			if(mserr=="est"){
-				index.mat[1,1:k] <- np+2
-			}
-			else{
-				index.mat[1,1:k] <- np+1
-			}
+			np <- k+2 #We have k regimes, but 1 sigma, plus the root value
+            index.mat[1,1:k] <- np+1
 			index.mat[2,1:k] <- 1
 			index.mat[3,1:k] <- 2:(np-1)
 			param.count<-np
@@ -283,15 +303,8 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 		}
 		if (model == "TrendyMS"){
 			index.mat <- matrix(0,3,k)
-			Rate.mat <- matrix(1, 3, k)
-			np= (k*2)+1 #We have k regimes and assume k trends and k sigmas, plus the root value
-			index <- matrix(TRUE,3,k)
-			if(mserr=="est"){
-				index.mat[1,1:k] <- np+2
-			}
-			else{
-				index.mat[1,1:k] <- np+1
-			}
+			np <- (k*2)+1 #We have k regimes and assume k trends and k sigmas, plus the root value
+            index.mat[1,1:k] <- np+1
 			index.mat[2,1:k] <- 1:k
 			index.mat[3,1:k] <- (k+1):(np-1)
 			param.count <- np
@@ -300,6 +313,8 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 		}
 	}
 
+    index.mat[is.na(index.mat)] <- param.count+1
+
 	if(warn==TRUE){
 		if(param.count > (ntips/10)){
 			warning("You might not have enough data to fit this model well", call.=FALSE, immediate.=TRUE)
@@ -308,43 +323,59 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 
 	#Likelihood function for estimating model parameters
 	dev <- function(p, index.mat, edges, mserr, trendy, get.root.theta){
-        
 		if(trendy == 0){
+            
             p <- exp(p)
             Rate.mat[] <- c(p, 1e-10)[index.mat]
-
-            N <- length(x[,1])
-            root.par.index <- length(p)
-            V <- varcov.ou(phy, edges, Rate.mat, root.state=root.state, simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight, assume.station=root.station, shift.point=shift.point)
-            
-            if (any(is.nan(diag(V))) || any(is.infinite(diag(V)))) return(1000000)
-            if(mserr=="known"){
-                diag(V) <- diag(V)+(data[,3]^2)
-            }
-            if(mserr=="est"){
-                root.par.index = length(p) - 1
-                diag(V) <- diag(V) + (p[length(p)])
-            }
-            
             if(get.root.theta == TRUE){
                 W <- weight.mat(phy, edges, Rate.mat, root.state=root.state, simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight, assume.station=FALSE, shift.point=shift.point)
             }else{
                 W <- weight.mat(phy, edges, Rate.mat, root.state=root.state, simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight, assume.station=TRUE, shift.point=shift.point)
             }
-
-            theta <- Inf
-			try(theta <- pseudoinverse(t(W)%*%pseudoinverse(V)%*%W)%*%t(W)%*%pseudoinverse(V)%*%x, silent=TRUE)
-			if(any(theta==Inf)){
-				return(10000000)
-			}
-			DET <- sum(log(abs(Re(diag(qr(V)$qr)))))
-			#However, sometimes this fails (not sure yet why) so I just toggle between this and another approach:
-			if(!is.finite(DET)){
-				DET <- determinant(V, logarithm=TRUE)
-				logl <- -.5*(t(x-W%*%theta)%*%pseudoinverse(V)%*%(x-W%*%theta))-.5*as.numeric(DET$modulus)-.5*(N*log(2*pi))
-			}else{
-				logl <- -.5*(t(x-W%*%theta)%*%pseudoinverse(V)%*%(x-W%*%theta))-.5*as.numeric(DET)-.5*(N*log(2*pi))
-			}
+            
+            if(algorithm == "invert"){
+                N <- length(x[,1])
+                V <- varcov.ou(phy, edges, Rate.mat, root.state=root.state, simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight, assume.station=root.station, shift.point=shift.point)
+                
+                if (any(is.nan(diag(V))) || any(is.infinite(diag(V)))) return(1000000)
+                if(mserr=="known"){
+                    diag(V) <- diag(V)+(data[,3]^2)
+                }
+                
+                theta <- Inf
+                try(theta <- pseudoinverse(t(W)%*%pseudoinverse(V)%*%W)%*%t(W)%*%pseudoinverse(V)%*%x, silent=TRUE)
+                if(any(theta==Inf)){
+                    return(10000000)
+                }
+                DET <- sum(log(abs(Re(diag(qr(V)$qr)))))
+                #However, sometimes this fails (not sure yet why) so I just toggle between this and another approach:
+                if(!is.finite(DET)){
+                    DET <- determinant(V, logarithm=TRUE)
+                    logl <- -.5*(t(x-W%*%theta)%*%pseudoinverse(V)%*%(x-W%*%theta))-.5*as.numeric(DET$modulus)-.5*(N*log(2*pi))
+                }else{
+                    logl <- -.5*(t(x-W%*%theta)%*%pseudoinverse(V)%*%(x-W%*%theta))-.5*as.numeric(DET)-.5*(N*log(2*pi))
+                }
+            }
+            if(algorithm == "three.point"){
+                pars <- matrix(c(Rate.mat[3,], Rate.mat[2,], Rate.mat[1,]), dim(Rate.mat)[2], 3, dimnames = list(levels(tot.states), c("opt", "sig", "alp")))
+                if(get.root.theta == TRUE){
+                    root.par.index <- length(p)
+                    theta0 <- p[root.par.index]
+                    expected.vals <- colSums(t(W) * c(theta0, pars[,1]))
+                    names(expected.vals) <- phy$tip.label
+                }else{
+                    expected.vals <- colSums(t(W) * pars[,1])
+                    names(expected.vals) <- phy$tip.label
+                }
+                transformed.tree <- transformPhy(phy, map, pars)
+                comp <- NA
+                try(comp <- phylolm::three.point.compute(transformed.tree$tree, x, expected.vals, transformed.tree$diag), silent=TRUE)
+                if(is.na(comp[1])){
+                    return(10000000)
+                }else{
+                    logl <- -as.numeric(Ntip(phy) * log(2 * pi) + comp$logd + (comp$PP - 2 * comp$QP + comp$QQ))/2
+                }
+            }
 		}else{
             p <- exp(p)
             Rate.mat[] <- c(p, 1e-10)[index.mat]
@@ -354,10 +385,6 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
             if (any(is.nan(diag(V))) || any(is.infinite(diag(V)))) return(1000000)
             if(mserr=="known"){
                 diag(V)<-diag(V)+(data[,3]^2)
-            }
-            if(mserr=="est"){
-                root.par.index <- length(p) - 1
-                diag(V)<-diag(V)+(p[length(p)])
             }
 			E_a <- Inf
 			try(E_a <- expected.trendy(phy, edges, Rate.mat, root.state=root.state, simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight, root.value=p[root.par.index], shift.point=shift.point))
@@ -386,37 +413,29 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 	}
 
 	lb = -20
-	ub = 20
+	ub = log(100)
 	lower = rep(-20, np)
-	upper = rep(20, np)
+	upper = rep(log(100), np)
 
-	#opts <- list("algorithm"="NLOPT_LN_SBPLX", "maxeval"="1000000", "ftol_rel"=.Machine$double.eps^0.5)
+    if(scaleHeight==TRUE){
+        phy$edge.length <- phy$edge.length/Tmax
+        Tmax <- 1
+    }else{
+        Tmax <- max(branching.times(phy))
+    }
+
+    if(algorithm == "three.point"){
+        if(simmap.tree == FALSE){
+            map <- getMapFromNode(phy, tip.states, int.states, shift.point)
+        }else{
+            map <- phy$maps
+        }
+    }
 
 	if(model == "OU1" | model == "OUM" | model == "OUMV" | model == "OUMA" | model == "OUMVA"){
-		if(scaleHeight==TRUE){
-			d <- max(diag(vcv.phylo(phy)))
-			phy$edge.length <- (phy$edge.length/d)
-		}
-		C.mat <- vcv.phylo(phy)
-		a <- as.numeric(colSums(solve(C.mat))%*%x/sum(solve(C.mat)))
-		sig <- as.numeric(t(x-a)%*%solve(C.mat)%*%(x-a)/n)
-		init.np=2
-		init.lower = rep(lb, init.np)
-		init.upper = rep(ub, init.np)
-		init.index.mat<-matrix(0,2,k)
-		init.index.mat[1,1:k]<-1
-		init.index.mat[2,1:k]<-2
-		if(model == "OU1"){
-            edges.tmp=edges
-            edges.tmp[,6] <- rep(1, length(edges.tmp[,1]))
-		}else{
-			edges.tmp=edges
-		}
-
-		#Initial value for alpha is just the half life based on the entire length of the tree:
+        #Initial value for alpha is just the half life based on the entire length of the tree:
         if(is.null(starting.vals)){
-            init <- nloptr(x0=log(c(log(2)/max(branching.times(phy)),sig)), eval_f=dev, lb=init.lower, ub=init.upper, opts=opts, index.mat=init.index.mat, edges=edges.tmp, mserr=mserr, trendy=trendy, get.root.theta=FALSE)
-            init.ip <- c(exp(init$solution[1]), exp(init$solution[2]))
+            init.ip <- c(log(2)/Tmax, mean(pic(x, phy)^2))
         }else{
             init.ip <- starting.vals
         }
@@ -424,60 +443,69 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 			ip <- init.ip
 		}
 		if(model=="OUMV" | model=="OUMA" | model=="OUM"){
-			ip <- c(rep(init.ip[1],length(unique(index.mat[1,]))),rep(init.ip[2],length(unique(index.mat[2,]))))
+			ip <- c(rep(init.ip[1],length(unique(index.mat[1,]))), rep(init.ip[2],length(unique(index.mat[2,]))))
 		}
 		if(model=="OUMVA"){
 			ip <- c(rep(init.ip,k))
 		}
-		if(mserr=="est"){
-			ip <- c(ip,sig)
-			lower = c(lower,0)
-			upper = c(upper,ub)
-		}
+        if(algorithm == "three.point"){
+            if(model == "OU1"){
+                ip <- c(ip, mean(x))
+            }else{
+                means.by.regime <- with(data, tapply(data[,2], data[,1], mean))
+                names(means.by.regime) <- NULL
+                ip <- c(ip, means.by.regime)
+            }
+            lower <- c(lower, rep(-20, k))
+            upper <- c(upper, rep( 20, k))
+            if(get.root.theta == TRUE){
+                ip <- c(ip, means.by.regime[root.state])
+                lower <- c(lower, -20)
+                upper <- c(upper, 20)
+            }
+        }
 		if(quiet==FALSE){
 			cat("Finished. Begin thorough search...", "\n")
 		}
-		out = nloptr(x0=log(ip), eval_f=dev, lb=lower, ub=upper, opts=opts, index.mat=index.mat, edges=edges, mserr=mserr, trendy=trendy, get.root.theta=get.root.theta)
+		out <- nloptr(x0=log(ip), eval_f=dev, lb=lower, ub=upper, opts=opts, index.mat=index.mat, edges=edges, mserr=mserr, trendy=trendy, get.root.theta=get.root.theta)
 	}
 	else{
-		if(scaleHeight==TRUE){
-			d <- max(diag(vcv.phylo(phy)))
-			phy$edge.length<-(phy$edge.length/d)
-		}
-		#Starting values follow from phytools:
-		C.mat <- vcv.phylo(phy)
-		a <- as.numeric(colSums(solve(C.mat))%*%x/sum(solve(C.mat)))
         if(is.null(starting.vals)){
-            sig <- as.numeric(t(x-a)%*%solve(C.mat)%*%(x-a)/n)
+            sig <- mean(pic(x, phy)^2)
         }else{
             sig <- starting.vals
         }
 		#####################
 		if(model=="BMS"){
-			ip <- rep(sig,k)
+			ip <- rep(sig, k)
+            if(get.root.theta == TRUE){
+                ip <- c(ip, mean(x))
+                lower <- c(lower, -20)
+                upper <- c(upper, 20)
+            }
 		}
 		else{
 			if(model=="BM1"){
 				ip <- sig
+                if(get.root.theta == TRUE){
+                    ip <- c(ip, mean(x))
+                    lower <- c(lower, -20)
+                    upper <- c(upper, 20)
+                }
 			}
 			if(model=="TrendyM"){
 				#We assume that the starting trend values are zero:
-				ip <- c(sig, rep(exp(-20), param.count-2), a)
+				ip <- c(sig, rep(exp(-20), param.count-2), mean(x))
 				lower <- c(lb,rep(-20, param.count-2), -20)
 				upper <- c(ub,rep(20, param.count-2), 20)
 			}
 			if(model == "TrendyMS"){
 				#We assume that the starting trend values are zero:
-				ip <- c(rep(sig, k), rep(exp(-20), (np-1) - k), a)
+				ip <- c(rep(sig, k), rep(exp(-20), (np-1) - k), mean(x))
 				lower <- c(rep(lb,k), rep(-20, (np-1) - k), -20)
 				upper <- c(rep(ub,k), rep(20, (np-1) - k), 20)
 			}
 		}
-		if(mserr=="est"){
-			ip <- c(ip,sig)
-			lower <- c(lower,0)
-			upper <- c(upper,10)
-		}
 		if(quiet==FALSE){
 			cat("Finished. Begin thorough search...", "\n")
 		}
@@ -490,7 +518,6 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 	
     #Takes estimated parameters from dev and calculates theta for each regime:
 	dev.theta <- function(p, index.mat, edges=edges, mserr=mserr){
-		tmp<-NULL
 		Rate.mat[] <- c(p, 1e-10)[index.mat]
 		N <- length(x[,1])
 		V <- varcov.ou(phy, edges, Rate.mat, root.state=root.state, simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight, assume.station=root.station, shift.point=shift.point)
@@ -504,29 +531,13 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
         if(mserr=="known"){
             diag(V) <- diag(V)+(data[,3]^2)
 		}
-		if(mserr=="est"){
-			diag(V) <- diag(V)+(p[length(p)])
-		}
 		theta <- pseudoinverse(t(W)%*%pseudoinverse(V)%*%W)%*%t(W)%*%pseudoinverse(V)%*%x
         
-        #Calculates the hat matrix:
-		#H.mat<-W%*%pseudoinverse(t(W)%*%pseudoinverse(V)%*%W)%*%t(W)%*%pseudoinverse(V)
 		#Standard error of theta -- uses pseudoinverse to overcome singularity issues
 		se <- sqrt(diag(pseudoinverse(t(W)%*%pseudoinverse(V)%*%W)))
-        tmp$res <- W%*%theta-x
 		#Joins the vector of thetas with the vector of standard errors into a 2 column matrix for easy extraction at the summary stage
-		tmp$theta.est <- cbind(theta,se)
-
-        #Calculates R-sq according to Judge et al. 1985 eq. 2.3.16:
-        residuals <- tmp$res
-        fitted.vals <- W%*%theta
-        Y <- residuals + fitted.vals
-        one <- matrix(1, length(Y), 1)
-        a <- as.numeric(pseudoinverse(t(one) %*% pseudoinverse(V) %*% one) %*% (t(one) %*% pseudoinverse(V) %*% Y))
-        Rsq <- 1-(t(residuals) %*% pseudoinverse(V) %*% residuals) / (t(Y-a) %*% pseudoinverse(V) %*% (Y-a))
-		tmp$Rsq <- Rsq
-        tmp$W <- W
-        return(tmp)
+		theta.est <- cbind(theta,se)
+        return(theta.est)
 	}
 
 	#Informs the user that the summarization has begun, output model-dependent and dependent on whether the root theta is to be estimated
@@ -537,21 +548,26 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 	if(model == "TrendyM" | model == "TrendyMS"){
 		theta<-NULL
 		root.est <- out$solution[length(out$solution)]
-		theta$theta.est <- matrix(root.est, k+1, 2)
-		theta$theta.est[,2] <- 0
-		theta$res <- expected.trendy(phy, edges, Rate.mat, root.state=root.state, simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight, root.value=root.est, shift.point=shift.point) - x
+		theta <- matrix(root.est, k+1, 2)
+		theta[,2] <- 0
 	}else{
-		theta <- dev.theta(out$solution, index.mat, edges, mserr)
+        if(algorithm == "invert"){
+            theta <- dev.theta(out$solution, index.mat, edges, mserr)
+        }
 	}
-    
-    
+
 	#Calculates the Hessian for use in calculating standard errors and whether the maximum likelihood solution was found
 	if(diagn==TRUE){
 		h <- hessian(x=log(out$solution), func=dev, index.mat=index.mat, edges=edges, mserr=mserr, trendy=trendy, get.root.theta=get.root.theta)
 		#Using the corpcor package here to overcome possible NAs with calculating the SE
 		solution <- matrix(out$solution[index.mat], dim(index.mat))
 		solution.se <- matrix(sqrt(diag(pseudoinverse(h)))[index.mat], dim(index.mat))
-		rownames(solution) <- rownames(solution.se) <- rownames(index.mat) <- c("alpha","sigma.sq")
+        if(algorithm == "invert"){
+            rownames(solution) <- rownames(solution.se) <- rownames(index.mat) <- c("alpha","sigma.sq")
+        }
+        if(algorithm == "three.point"){
+            rownames(solution) <- rownames(solution.se) <- rownames(index.mat) <- c("alpha","sigma.sq", "theta")
+        }
 		if(simmap.tree==FALSE){
 			colnames(solution) <- colnames(solution.se) <- levels(tot.states)
 		}
@@ -563,19 +579,24 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 		#If eigenvect is TRUE then the eigenvector and index matrix will appear in the list of objects
 		eigval<-signif(hess.eig$values,2)
 		eigvect<-round(hess.eig$vectors, 2)
-		if(mserr=="est"){
-			mserr.est<-out$solution[length(out$solution)]
-			param.count<-param.count+1
-		}
-		else{
-			mserr.est<-NULL
-		}
         
         if(get.root.theta == TRUE){
+            if(model == "BM1" | model == "BMS"){
+                W <- matrix(0, Ntip(phy), k)
+                W[,1] <- 1
+            }else{
+                W <- weight.mat(phy, edges, Rate.mat=solution, root.state=root.state, simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight, assume.station=FALSE, shift.point=shift.point)
+            }
             rates <- cbind(NA, solution, NA)
-            thetas <- cbind(t(theta$theta.est[,1]), NA)
-            rates <- rbind(rates, thetas)
-            weights <- cbind(theta$W, data[,1])
+            if(algorithm == "invert"){
+                thetas <- cbind(t(theta[,1]), NA)
+                rates <- rbind(rates, thetas)
+            }else{
+                theta.est <- solution[3,]
+                se <- rep(NA,length(theta))
+                theta <- cbind(theta.est,se)
+            }
+            weights <- cbind(W, data[,1])
             regime.weights <- rbind(rates, weights)
             rownames(regime.weights) <- c("alpha", "sigma.sq", "theta", phy$tip.label)
             colnames(regime.weights) <- c("Root", levels(tot.states), "Tip_regime")
@@ -583,17 +604,24 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
             if(model=="TrendyM" | model=="TrendyMS"){
                 regime.weights = NULL
             }else{
+                W <- weight.mat(phy, edges, Rate.mat=solution, root.state=root.state, simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight, assume.station=TRUE, shift.point=shift.point)
                 rates <- cbind(solution, NA)
-                thetas <- cbind(t(theta$theta.est[,1]), NA)
-                rates <- rbind(rates, thetas)
-                weights <- cbind(theta$W, data[,1])
+                if(algorithm == "invert"){
+                    thetas <- cbind(t(theta[,1]), NA)
+                    rates <- rbind(rates, thetas)
+                }else{
+                    theta.est <- solution[3,]
+                    se <- rep(NA,length(theta))
+                    theta <- cbind(theta.est,se)
+                }
+                weights <- cbind(W, data[,1])
                 regime.weights <- rbind(rates, weights)
                 rownames(regime.weights) <- c("alpha", "sigma.sq", "theta", phy$tip.label)
                 colnames(regime.weights) <- c(levels(tot.states), "Tip_regime")
             }
         }
 
-		obj = list(loglik = loglik, AIC = -2*loglik+2*param.count, AICc=-2*loglik+(2*param.count*(ntips/(ntips-param.count-1))), BIC=-2*loglik + log(ntips) * param.count, model=model, param.count=param.count, solution=solution, mserr.est=mserr.est, theta=theta$theta.est, solution.se=solution.se, tot.states=tot.states, index.mat=index.mat, simmap.tree=simmap.tree, root.age=root.age, shift.point=shift.point, opts=opts, data=data, phy=phy, root.station=root.station, scaleHeight=scaleHeight, starting.vals=starting.vals, lb=lower, ub=upper, iterations=out$iterations, res=theta$res, rsq=theta$Rsq, get.root.theta=get.root.theta, regime.weights=regime.weights, eigval=eigval, eigvect=eigvect, new.start=out$new.start)
+		obj = list(loglik = loglik, AIC = -2*loglik+2*param.count, AICc=-2*loglik+(2*param.count*(ntips/(ntips-param.count-1))), BIC = -2*loglik + log(ntips) * param.count, model=model, param.count=param.count, solution=solution, theta=theta, solution.se=solution.se, tot.states=tot.states, index.mat=index.mat, simmap.tree=simmap.tree, root.age=root.age, shift.point=shift.point, opts=opts, data=data, phy=phy, root.station=root.station, scaleHeight=scaleHeight, starting.vals=starting.vals, lb=lower, ub=upper, iterations=out$iterations, get.root.theta=get.root.theta, regime.weights=regime.weights, eigval=eigval, eigvect=eigvect, new.start=out$new.start, algorithm=algorithm)
 
 	}
 	if(diagn==FALSE){
@@ -601,7 +629,11 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 		if(model=="TrendyM" | model=="TrendyMS"){
 			rownames(solution) <- rownames(index.mat) <- c("alpha","sigma.sq","trend")
 		}else{
-			rownames(solution) <- rownames(index.mat) <- c("alpha","sigma.sq")
+            if(algorithm == "three.point"){
+                rownames(solution) <- rownames(index.mat) <- c("alpha","sigma.sq", "theta")
+            }else{
+                rownames(solution) <- rownames(index.mat) <- c("alpha","sigma.sq")
+            }
 		}
 		if(simmap.tree==FALSE){
 			colnames(solution) <- levels(tot.states)
@@ -618,10 +650,22 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 		}
         
         if(get.root.theta == TRUE){
+            if(model == "BM1" | model == "BMS"){
+                W <- matrix(0, Ntip(phy), k+1)
+                W[,1] <- 1
+            }else{
+                W <- weight.mat(phy, edges, Rate.mat=solution, root.state=root.state, simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight, assume.station=FALSE, shift.point=shift.point)
+            }
             rates <- cbind(NA, solution, NA)
-            thetas <- cbind(t(theta$theta.est[,1]), NA)
-            rates <- rbind(rates, thetas)
-            weights <- cbind(theta$W, data[,1])
+            if(algorithm == "invert"){
+                thetas <- cbind(t(theta[,1]), NA)
+                rates <- rbind(rates, thetas)
+            }else{
+                theta.est <- solution[3,]
+                se <- rep(NA,length(theta.est))
+                theta <- cbind(theta.est,se)
+            }
+            weights <- cbind(W, data[,1])
             regime.weights <- rbind(rates, weights)
             rownames(regime.weights) <- c("alpha", "sigma.sq", "theta", phy$tip.label)
             colnames(regime.weights) <- c("Root", levels(tot.states), "Tip_regime")
@@ -629,17 +673,24 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
             if(model=="TrendyM" | model=="TrendyMS"){
                 regime.weights = NULL
             }else{
+                W <- weight.mat(phy, edges, Rate.mat=solution, root.state=root.state, simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight, assume.station=TRUE, shift.point=shift.point)
                 rates <- cbind(solution, NA)
-                thetas <- cbind(t(theta$theta.est[,1]), NA)
-                rates <- rbind(rates, thetas)
-                weights <- cbind(theta$W, data[,1])
+                if(algorithm == "invert"){
+                    thetas <- cbind(t(theta[,1]), NA)
+                    rates <- rbind(rates, thetas)
+                }else{
+                    theta.est <- solution[3,]
+                    se <- rep(NA,length(theta.est))
+                    theta <- cbind(theta.est,se)
+                }
+                weights <- cbind(W, data[,1])
                 regime.weights <- rbind(rates, weights)
                 rownames(regime.weights) <- c("alpha", "sigma.sq", "theta", phy$tip.label)
                 colnames(regime.weights) <- c(levels(tot.states), "Tip_regime")
             }
         }
 
-        obj = list(loglik = loglik, AIC = -2*loglik+2*param.count,AICc=-2*loglik+(2*param.count*(ntips/(ntips-param.count-1))), BIC=-2*loglik + log(ntips) * param.count, model=model, param.count=param.count, solution=solution, mserr.est=mserr.est, theta=theta$theta.est, tot.states=tot.states, index.mat=index.mat, simmap.tree=simmap.tree, root.age=root.age, shift.point=shift.point, opts=opts, data=data, phy=phy, root.station=root.station, scaleHeight=scaleHeight, starting.vals=starting.vals, lb=lower, ub=upper, iterations=out$iterations, res=theta$res, rsq=theta$Rsq, get.root.theta=get.root.theta, regime.weights=regime.weights, new.start=out$new.start)
+        obj = list(loglik = loglik, AIC = -2*loglik+2*param.count,AICc=-2*loglik+(2*param.count*(ntips/(ntips-param.count-1))), BIC=-2*loglik + log(ntips) * param.count, model=model, param.count=param.count, solution=solution, theta=theta, tot.states=tot.states, index.mat=index.mat, simmap.tree=simmap.tree, root.age=root.age, shift.point=shift.point, opts=opts, data=data, phy=phy, root.station=root.station, scaleHeight=scaleHeight, starting.vals=starting.vals, lb=lower, ub=upper, iterations=out$iterations, get.root.theta=get.root.theta, regime.weights=regime.weights, new.start=out$new.start, algorithm=algorithm)
 	}
 	class(obj)<-"OUwie"
 	return(obj)
@@ -648,9 +699,9 @@ OUwie <- function(phy,data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA
 
 print.OUwie<-function(x, ...){
 
-	ntips=Ntip(x$phy)
-	output<-data.frame(x$loglik,x$AIC,x$AICc,x$BIC,x$model,ntips, row.names="")
-	names(output)<-c("lnL","AIC","AICc","BIC","model","ntax")
+	ntips <- Ntip(x$phy)
+	output <- data.frame(x$loglik,x$AIC,x$AICc,x$BIC,x$model,ntips, row.names="")
+	names(output) <- c("lnL","AIC","AICc","BIC","model","ntax")
 	cat("\nFit\n")
 	print(output)
 	cat("\n")
@@ -680,7 +731,7 @@ print.OUwie<-function(x, ...){
 		}
         if (x$get.root.theta == FALSE){
             if (x$model == "OU1" | x$model == "OUM"| x$model == "OUMV"| x$model == "OUMA" | x$model == "OUMVA"){
-                param.est<- x$solution
+                param.est<- x$solution[1:2,]
                 theta.mat<-matrix(t(x$theta), 2, length(levels(x$tot.states)))
                 rownames(theta.mat)<-c("estimate", "se")
                 if(x$simmap.tree==FALSE){
@@ -696,13 +747,17 @@ print.OUwie<-function(x, ...){
                 print(theta.mat)
                 cat("\n")
                 cat("\nHalf life (another way of reporting alpha)\n")
-                print(log(2)/param.est['alpha',])
+                if(x$model == "OU1"){
+                    print(log(2)/param.est[1])
+                }else{
+                    print(log(2)/param.est['alpha',])
+                }
                 cat("\n")
             }
         }
         if (x$get.root.theta == TRUE){
             if (x$model == "OU1" | x$model == "OUM"| x$model == "OUMV"| x$model == "OUMA" | x$model == "OUMVA"){
-                param.est<- x$solution
+                param.est<- x$solution[1:2,]
                 theta.mat<-matrix(t(x$theta), 2, length(levels(x$tot.states))+1)
                 rownames(theta.mat)<-c("estimate", "se")
                 if(x$simmap.tree==FALSE){
@@ -718,7 +773,11 @@ print.OUwie<-function(x, ...){
                 print(theta.mat)
                 cat("\n")
                 cat("\nHalf life (another way of reporting alpha)\n")
-                print(log(2)/param.est['alpha',])
+                if(x$model == "OU1"){
+                    print(log(2)/param.est[1])
+                }else{
+                    print(log(2)/param.est['alpha',])
+                }
                 cat("\n")
             }
         }
